﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;

namespace WindowsFormsApp2
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// Auflisten aller Labels auf der Form,
        /// auch aus Child-Controls
        /// </summary>
        private void button1_Click(object sender, EventArgs e)
        {
            IEnumerable<Control> c = GetAllType(this, typeof(Label));
            // was damit tun:
            MessageBox.Show("Total Labelss: " + c.Count());
            foreach (Label lbl in c)
            {
                lbl.Text = "bla";
            }
        }

        /// <summary>
        /// Auflisten aller von UserControl abgeleiteten Controls auf der Form
        /// </summary>
        private void button2_Click(object sender, EventArgs e)
        {
            IEnumerable<Control> c = GetAllClasses(this, typeof(UserControl));
            // was damit tun:
            MessageBox.Show("Total UserControls: " + c.Count());
            foreach (UserControl ctrl in c)
            {
                UserControl1 ctrl1 = ctrl as UserControl1;
                UserControl2 ctrl2 = ctrl as UserControl2;

                if (ctrl1 != null) { ctrl1.SetText(); }
                if (ctrl2 != null) { ctrl2.SetText(); }
            }
        }

        /// <summary>
        /// Auflisten aller das Interface Interface1 implementierenden Controls auf der Form
        /// </summary>
        private void button3_Click(object sender, EventArgs e)
        {
            IEnumerable<Control> c = GetAllInterfaces(this, typeof(Interface1));
            // was damit tun:
            MessageBox.Show("Total Interfaces: " + c.Count());
            foreach (Interface1 ctrl in c)  // VB2019 macht dies automatisch
            {
                if (ctrl != null) { ctrl.SetTextEx(); }
            }
        }

        /// <summary>
        /// alle Objekte aufsammeln, deren aktuelle Instanz vom Type type ist
        /// </summary>
        /// <param name="control">das Parent-Control</param>
        /// <param name="type">der Ziel-Type</param>
        /// <returns>Auflistung der Controls</returns>
        private static IEnumerable<Control> GetAllType(Control control, Type type)
        {
            // alle Controls aufsammeln
            IEnumerable<Control> controls = control.Controls.Cast<Control>();
            // aus dieser nur die Ziel-Controls zurückgeben
            return controls.SelectMany(ctrl => GetAllType(ctrl, type))
                                      .Concat(controls)
                                      .Where(c => c.GetType() == type);
        }

        /// <summary>
        /// alle Objekte aufsammeln, die vom Type type abgeleitet sind
        /// </summary>
        /// <param name="control">das Parent-Control</param>
        /// <param name="cls">der Ziel-Type</param>
        /// <returns>Auflistung der Controls</returns>
        private static IEnumerable<Control> GetAllClasses(Control control, Type cls)
        {
            // alle Controls aufsammeln
            IEnumerable<Control> controls = control.Controls.Cast<Control>();
            // aus dieser nur die Ziel-Controls zurückgeben
            return controls.SelectMany(ctrl => GetAllClasses(ctrl, cls))
                                      .Concat(controls)
                                      .Where(c => c.TestClass(cls));
        }

        /// <summary>
        /// alle Objekte aufsammeln, die das Interface interf implementieren
        /// </summary>
        /// <param name="control">das Parent-Control</param>
        /// <param name="interf">das Ziel-Interface</param>
        private static IEnumerable<Control> GetAllInterfaces(Control control, Type interf)
        {
            // alle Controls aufsammeln
            IEnumerable<Control> controls = control.Controls.Cast<Control>();
            // aus dieser nur die Ziel-Controls zurückgeben
            return controls.SelectMany(ctrl => GetAllInterfaces(ctrl, interf))
                                      .Concat(controls)
                                      .Where(c => c.TestInterface(interf));
        }
    }
}
